==> relations-0.0.1.tar.gz: lib/utils/collision.dart <==
import 'package:flutter/material.dart';
import 'package:relations/bloc/bloc.dart';
import 'package:relations/utils/math_util.dart';
import 'dart:math' as math;
import '../models/graph_node.dart';

/// Contains utility functions for collision detection and response for rectangles.
class Collision {

  static bool checkAndResolve(GraphNode node1, GraphNode node2) {
    var collision = collide(node1, node2);
    bool result = collision[0];
    if (result == true) {

      Offset mtv = collision[1] * -1.0;
      resolve(node1, node2, mtv);
      return true;
    }
    return false;
  }

  static List<Offset> _getAxis(List<Offset> corners1, List<Offset> corners2) {
    return [
      MathUtil.normalize(corners1[1] - corners1[0]),
      MathUtil.normalize(corners1[3] - corners1[0]),
      MathUtil.normalize(corners2[1] - corners2[0]),
      MathUtil.normalize(corners2[3] - corners2[0]),
    ];
  }

  /// Collision check using the separated axis theorem (SAT).
  /// Returns a pair: the first value is a flag indicating whether
  /// the bounding boxes of nodes are collided (overlapped).
  /// The second value is the mtv (Minimum Translation Vector).
  static List collide(GraphNode node1, GraphNode node2) {
    /// Note: mtv = minimum translation vector
    /// array containing the mtvs for each axises
    List<Offset> mtvs = List<Offset>(4);

    var c1 = node1.globalBoundCoords;
    var c2 = node2.globalBoundCoords;

    List<Offset> axises = _getAxis(node1.globalBoundCoords, node2.globalBoundCoords);

    for (int k = 0; k < axises.length; k++) {

        var axis = axises[k];

        List<double> scalars1 = List<double>(4);
        List<double> scalars2 = List<double>(4);

        for (var i = 0; i < 4; i++) {
          scalars1[i] = MathUtil.dot(axis, c1[i]);
          scalars2[i] = MathUtil.dot(axis, c2[i]);
        }
        
        double s1max = scalars1.reduce(math.max);
        double s1min = scalars1.reduce(math.min);
        double s2max = scalars2.reduce(math.max);
        double s2min = scalars2.reduce(math.min);

        if (s2min >= s1max || s2max <= s1min)
        {
            return [false];
        }

        double overlap = s1max > s2max ? -(s2max - s1min) : (s1max - s2min);

        mtvs[k] = axis * overlap;
    }

    /// find minimum tranlsation vector
    int minIdx = 0;
    for (var i = 1; i < mtvs.length; i++) {
      double len = mtvs[i].distanceSquared;
      double minLen = mtvs[minIdx].distanceSquared;
      if (len < minLen) {
        minIdx = i;
      }
    } 
    return [true, mtvs[minIdx]];
  }

  static bool contains(Rect rect, Rect area) =>
            area.contains(rect.topLeft) &&
            area.contains(rect.topRight) && 
            area.contains(rect.bottomRight) && 
            area.contains(rect.bottomLeft);
  

  static void resolve(GraphNode node1, GraphNode node2, Offset mtv) {
    const int space = 20;

    if (mtv.dx < 0.0) {
      mtv = Offset(mtv.dx - space, mtv.dy);
    }    
    else if (mtv.dx > 0.0){
      mtv = Offset(mtv.dx + space, mtv.dy);
    }

    if (mtv.dy < 0.0) {
      mtv = Offset(mtv.dx, mtv.dy - space);
    }    
    else if (mtv.dy > 0.0){
      mtv = Offset(mtv.dx, mtv.dy + space);
    }

    const double factor = 1.0;
    final Offset node1NextPos = node1.position + mtv * factor;
    final Offset node2NextPos = node2.position + mtv * factor * -1;
    final node1NextRect = Rect.fromLTWH(node1NextPos.dx, node1NextPos.dy, node1.size.width, node1.size.height);
    final node2NextRect = Rect.fromLTWH(node2NextPos.dx, node2NextPos.dy, node2.size.width, node2.size.height);

    /// add padding to the mtv in the corresponding axis
    
    if (node1.fixed && !node2.fixed)
    {
      if (contains(node2NextRect, StageBloc.stageBounds)) {
        node2.position += mtv;
        return;
      }
      node1.position += mtv * -1;
      return;
    }

    if (node2.fixed && !node1.fixed)
    {
      if (contains(node1NextRect, StageBloc.stageBounds)) {
        node1.position += mtv;
        return;
      }
      node2.position += mtv * -1;
      return;
    }
    
    if (contains(node1NextRect, StageBloc.stageBounds)) {
      node1.position += mtv * factor;
    }

    if (contains(node2NextRect, StageBloc.stageBounds)) {
      node2.position += mtv * -factor;
    }

  }
}

==> relations-0.0.3.tar.gz: lib/utils/collision.dart <==
import 'package:flutter/material.dart';
import 'package:relations/bloc/bloc.dart';
import 'package:relations/utils/math_util.dart';
import 'dart:math' as math;
import '../models/graph_node.dart';

/// Contains utility functions for collision detection and response for rectangles.
class Collision {
  static bool checkAndResolve(GraphNode node1, GraphNode node2) {
    var collision = collide(node1, node2);
    bool result = collision[0];
    if (result == true) {
      Offset mtv = collision[1] * -1.0;
      resolve(node1, node2, mtv);
      return true;
    }
    return false;
  }

  static List<Offset> _getAxis(List<Offset> corners1, List<Offset> corners2) {
    return [
      MathUtil.normalize(corners1[1] - corners1[0]),
      MathUtil.normalize(corners1[3] - corners1[0]),
      MathUtil.normalize(corners2[1] - corners2[0]),
      MathUtil.normalize(corners2[3] - corners2[0]),
    ];
  }

  /// Collision check using the separated axis theorem (SAT).
  /// Returns a pair: the first value is a flag indicating whether
  /// the bounding boxes of nodes are collided (overlapped).
  /// The second value is the mtv (Minimum Translation Vector).
  static List collide(GraphNode node1, GraphNode node2) {
    /// Note: mtv = minimum translation vector
    /// array containing the mtvs for each axises
    List<Offset> mtvs = List<Offset>(4);

    var c1 = node1.globalBoundCoords;
    var c2 = node2.globalBoundCoords;

    List<Offset> axises =
        _getAxis(node1.globalBoundCoords, node2.globalBoundCoords);

    for (int k = 0; k < axises.length; k++) {
      var axis = axises[k];

      List<double> scalars1 = List<double>(4);
      List<double> scalars2 = List<double>(4);

      for (var i = 0; i < 4; i++) {
        scalars1[i] = MathUtil.dot(axis, c1[i]);
        scalars2[i] = MathUtil.dot(axis, c2[i]);
      }

      double s1max = scalars1.reduce(math.max);
      double s1min = scalars1.reduce(math.min);
      double s2max = scalars2.reduce(math.max);
      double s2min = scalars2.reduce(math.min);

      if (s2min >= s1max || s2max <= s1min) {
        return [false];
      }

      double overlap = s1max > s2max ? -(s2max - s1min) : (s1max - s2min);

      mtvs[k] = axis * overlap;
    }

    /// find minimum tranlsation vector
    int minIdx = 0;
    for (var i = 1; i < mtvs.length; i++) {
      double len = mtvs[i].distanceSquared;
      double minLen = mtvs[minIdx].distanceSquared;
      if (len < minLen) {
        minIdx = i;
      }
    }
    return [true, mtvs[minIdx]];
  }

  static bool contains(Rect rect, Rect area) =>
      area.contains(rect.topLeft) &&
      area.contains(rect.topRight) &&
      area.contains(rect.bottomRight) &&
      area.contains(rect.bottomLeft);

  static void resolve(GraphNode node1, GraphNode node2, Offset mtv) {
    const int space = 20;

    if (mtv.dx < 0.0) {
      mtv = Offset(mtv.dx - space, mtv.dy);
    } else if (mtv.dx > 0.0) {
      mtv = Offset(mtv.dx + space, mtv.dy);
    }

    if (mtv.dy < 0.0) {
      mtv = Offset(mtv.dx, mtv.dy - space);
    } else if (mtv.dy > 0.0) {
      mtv = Offset(mtv.dx, mtv.dy + space);
    }

    const double factor = 1.0;
    final Offset node1NextPos = node1.position + mtv * factor;
    final Offset node2NextPos = node2.position + mtv * factor * -1;
    final node1NextRect = Rect.fromLTWH(
        node1NextPos.dx, node1NextPos.dy, node1.size.width, node1.size.height);
    final node2NextRect = Rect.fromLTWH(
        node2NextPos.dx, node2NextPos.dy, node2.size.width, node2.size.height);

    /// add padding to the mtv in the corresponding axis

    if (node1.fixed && !node2.fixed) {
      if (contains(node2NextRect, StageBloc.stageBounds)) {
        node2.position += mtv;
        return;
      }
      node1.position += mtv * -1;
      return;
    }

    if (node2.fixed && !node1.fixed) {
      if (contains(node1NextRect, StageBloc.stageBounds)) {
        node1.position += mtv;
        return;
      }
      node2.position += mtv * -1;
      return;
    }

    if (contains(node1NextRect, StageBloc.stageBounds)) {
      node1.position += mtv * factor;
    }

    if (contains(node2NextRect, StageBloc.stageBounds)) {
      node2.position += mtv * -factor;
    }
  }
}
